unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  uSP5LazV2,  Classes, SysUtils, FileUtil, LResources,
  Forms, Controls, Graphics, Dialogs, StdCtrls, ComCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    BAcquisition: TButton;
    BAcquisitionPoint: TButton;
    BacqPtsMultiples: TButton;
    BEmissionUContinue: TButton;
    BEmissionSinus: TButton;
    BArretEmission: TButton;
    BEmissionBologic: TButton;
    BlectureBologic: TButton;
    Edit1: TEdit;
    Memo1: TMemo;
    TrackBar1: TTrackBar;
    procedure BacqPtsMultiplesClick(Sender: TObject);
    procedure BAcquisitionClick(Sender: TObject);
    procedure BAcquisitionPointClick(Sender: TObject);
    procedure BArretEmissionClick(Sender: TObject);
    procedure BEmissionBologicClick(Sender: TObject);
    procedure BEmissionSinusClick(Sender: TObject);
    procedure BEmissionUContinueClick(Sender: TObject);
    procedure BlectureBologicClick(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);

  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

{ TForm1 }


procedure TForm1.BAcquisitionClick(Sender: TObject);
(* Placez ici les déclarations de variables locales *)
Var    mesure1 : word ;
voie : byte ;
s : string ; (* chaîne de caractères qui servira à la mise en forme de l’affichage du résultat *)

U : single ; (* valeur à virgule flottante destinée à stocker la valeur physique de la tension mesurée *)


begin
(* placez ici le code exemple *)

          Voie := 0 ;
          SP5.calibreVoieA ( voie, cal_05V );
          SP5.activeVoieA ( voie );
          SP5.acquisitionVoieUnique ( voie, mesure1) ;
          memo1.lines.add('Mesure sur l''entrée '+ nomsEA[voie]);

          s := intToStr(mesure1);
          s := ' valeur entière:  v = ' + s;
          memo1.lines.add(s);

          u := SP5. ux(voie, mesure1);
          s := floatToStrF(u, ffgeneral, 4 , 2);
          s := ' tension :        u = ' + s + ' V';
          memo1.lines.add(s);

end;

procedure TForm1.BacqPtsMultiplesClick(Sender: TObject);
var kV,    	(* compteur pour les voies *)
    kP,    	(* compteur des points dans un tour *)
    numPt, 	(* numéro de point en cours de traitement *)
    tp     	(* valeur de remplissage de tampon *)
         : integer;
    TamponSP5:  (* tableau recevant les valeurs acquises lors d'un tour *)
                TTamponSP5;

 (* paramètres pour fixer les idées  *)
    dureeTotale : single;
    nbPts,  (* nombre de points à “acquérir” *)
    nbPT   (* nombre de points du tampon de récupération des valeurs *)
            : integer;

 (* auxiliaires *)
    vNum: Word;
    u1,t1: single;
    AcquisitionTerminee: boolean;
    s1  (* chaîne servant d'outil de mise en forme pour l'affichage des résultats *)
                : string;


begin

 (* effacement du memo *)
    memo1.lines.add(' ');

(* on place un titre dans le memo *)

    memo1.lines.add( 'Acquisition monocoup');

(* choix des paramètres, à faire varier pour les tests  *)

    dureeTotale := 100e-3;   (* 100 ms *)
    nbPts       := 500;
    nbPT        := 50;
(* nombre de points souhaité dans le tampon. Minimum : 1 ;  maximum : 256 / nombre de voies actives.  Avec ce réglage (* nbPT := 50 *), et sachant que nous activerons 2 entrées, le tampon contiendra 100 valeurs, dans l’ordre temporel :
vneEA0(0) ; vneEA3(0) ; vneEA0(Dt) ; vneEA3(Dt) ; vneEA0(2*Dt) ; vneEA3(2*Dt) … vneEA0((50-1) * Dt) ; vneEA3((50-1)*Dt)
Le contenu du tampon correspond à un temps d'acquisition égal à :
dureeTotale / nbPts * nbPtsDansTampon
Le traitement des valeurs d’un tampon se fait lors d’un « tour » ou boucle. La valeur nbTP conditionne ainsi le temps réel: pour avoir un vrai temps réel,  il faut que le nombre de points du tampon puisse être traité par le programme pendant  la durée correspondante, de manière à être traité avant la fin de l'acquisition du tour suivant  *)

(* remise à l'état standard de SYSAM SP5 (facultatif) *)
    SP5.Reinitialisation;

(* activation des voies EA0 et EA3 et réglage des calibres  10V *)
    SP5.ActiveVoieA( EA0);
    SP5.CalibreVoieA( EA0, cal_10V);

    SP5.ActiveVoieA( EA3);
    SP5.CalibreVoieA( EA3, cal_10V);

(* réglage de la durée totale et du nombre de points à recueillir *)
    SP5.DureesEtPointsAcq( dureeTotale, nbPts);

(* réglage du nombre de points dans le tampon *)
    SP5.NbPtsDansTamponAcq(nbPT);

(* préparation diverses *)
    acquisitionTerminee := false;
(* mise à zéro du compteur de points *)
    numPt   := 0;

(* déclenchement *)
    SP5.declencheAcqMonocoup;

(* BOUCLE DE RECUPERATION DES VALEURS en temps presque réel
(une boucle = 1 tour = récupération et traitement de nbPtsTampon points.
« LireTamponAcq » renvoie une valeur différente de zéro à chaque fois qu'il s'est écoulé le temps correspondant au remplissage du tampon et que SYSAM a acquis un nombre de points contenu dans le tampon *)

    repeat
      tp := SP5.LireTamponAcq(TamponSP5);
      if tp > 0  then
        begin
(* à l'instant où tp > 0, on sait que le tampon a été rempli de nbPT mesures renvoyées au programme utilisateur dans le tableau TamponSP5 *)

(* TRAITEMENT DES VALEURS récupérées en temps quasi-réel.
Comme exemple, un simple ajout des valeurs récupérées dans le memo1, avec une mise en forme sommaire. Ce traitement n’est pas très rapide : l’ajout de lignes dans un mémo est lent *)

(* mise à zéro du compteur des points du tour *)
            kP := 0;
     (* boucle sur les points du tampon *)
            while     (kP    < SP5.nbPtsTampon) (* test de non fin de tampon *)
               and (numPt < SP5.nbPtsAcq   ) (* test de non fin d'acquisition *)
            do
              begin
       (* préparation de la chaîne de caractères s1 correspondant au point numPt *)
                      s1 :=  intToStr( numPt );
                   (* instant correspondant *)
                      t1 := numPt * SP5.DtAcq;
                      s1 := s1 + ' - t = '+ floatToStrF(t1, ffgeneral, 5,2);
                      memo1.lines.add( s1);

                   (* valeur numérique acquise *)
                      vNum := TamponSP5[ kP * SP5.nbVoiesActives];
                      s1 := ' - EA0  --> ' + IntToStr( vNum );
                   (* valeur de la tension mesurée *)
                      u1 := SP5.Ux( EA0 , vNum);
                   (* ajout de la valeur de tension dans la chaîne *)
                      s1 := s1 + ' ---- u = ' + floatToStrF(u1, ffgeneral, 5,2) + ' V';
                      memo1.lines.add( s1);

                   (* valeur numérique acquise *)
                      vNum := TamponSP5[ kP * SP5.nbVoiesActives + 1];
                      s1 := ' – EA3  --> ' + IntToStr( vNum);
                   (* valeur de la tension mesurée *)
                      u1 := SP5.Ux( EA3 , vNum);
                   (* ajout de la valeur de tension dans la chaîne *)
                      s1 := s1 + ' ---- u = ' + floatToStrF(u1, ffgeneral, 5,2) + ' V';
                      memo1.lines.add( s1);

                   (* compteur de points global : on passe au numéro suivant*)
                      inc(numPt);
                   (* compteur de points du tour : on passe au numéro suivant *)
                      inc(kP);
              end; (* while *)

(* test de fin d'acquisition. On se base sur le nombre de points souhaité  (nbPts) car
            1.  le tampon peut ne pas être rempli à nbPtsTampon Points au dernier tour, si le nombre total de points n'est pas un multiple entier de nbPtsTampon
            2.  le nombre de points renvoyé par SP5_nbPtsAcq et sur lequel est paramétré SYSAM peut être supérieur au nombre souhaité pour des raisons techniques *)

           if numPt >= nbPts - 1 then acquisitionTerminee := true;

        end;  (*  if tp > 0  then *)
    until acquisitionTerminee;

    SP5.arreteAcq;

 (* désactivation des voies *)
    for kV := 0 to 7 do SP5.DesactiveVoieA( kV );



end;

procedure TForm1.BAcquisitionPointClick(Sender: TObject);
var 	voie : integer;
    	s, s1: string;
        U : single;
        TM : TTabMesures ;

begin
  {Ecriture d’un titre :}
      memo1.lines.add('Acquisition d''une valeur sur les 8 entrées, avec le calibre +/-10 V');
  {Calibre +/- 10V sur les 8 entrées }
      for voie := EA0 to EA7 do SP5. calibreVoieA(voie, cal_10V);

  {activation de toutes les voies :}
      for voie := EA0 to EA7 do SP5. activeVoieA(voie);

{réalisation de l’acquisition :}
      SP5. acquisition(TM);
{lors de l’exécution du programme et après clic sur le bouton, à ce stade, le tableau TM contient une VNE dans chaque case}

{affichage des résultats dans le memo1 }
      for voie := EA0 to EA7 do {on répète pour les 8 entrées les instructions du bloc suivant}
           begin {début du bloc}
{construction d’une chaîne de caractères :}
              s := 'u('  +  nomsEA[voie]  +   ') = ';
{récupération de la tension :}
              u := SP5. ux(voie,  TM[voie]);
{conversion de la valeurs de la tension en chaîne de caractères :}
              s1 := floatToStrF(u , ffgeneral, 4, 2);
              s  := s + s1 + ' V';
{ajout de la chaîne dans le memo1 :}
              memo1.lines.add(s);
           end;  { fin du bloc }

end;

procedure TForm1.BArretEmissionClick(Sender: TObject);
begin
     SP5.ArreteEmission(SA1);
end;

procedure TForm1.BEmissionBologicClick(Sender: TObject);
begin
   memo1.lines.add(' ');
   memo1.lines.add('Mise à 1 des ports logiques B0 et B3');
   SP5.EmetPLB(0,1);
   SP5.EmetPLB(3,1);
   memo1.Lines.add('Vérifiez que les LED correspondantes sont allumées');

end;

procedure TForm1.BEmissionSinusClick(Sender: TObject);
 var
   k: integer;
   t,u : single ;
   (* caractéristiques de la tension à émettre *)

   per, ampl, phase: single;
   sortieUtilisee: integer;


begin
    if memo1.lines.count > 2000 then memo1.lines.clear;
   (* paramètres du signal *)
        per   := 2.00e-3 ;
        ampl  := 9 ;
        phase := pi/9;
        memo1.lines.add(' ');
        memo1.lines.add( 'Emission d''un signal sinusoïdal');
        memo1.lines.add( 'Fréquence: '+ floatToStrF(1/per, ffgeneral, 4,2)+ ' Hz');
        memo1.lines.add( 'Amplitude: '+ floatToStrF(ampl, ffgeneral, 4,2)+' V');
   (* choix de la sortie à utiliser *)
        sortieUtilisee := SA1 ;
   (* 1. On fixe les paramètres d'émission : 100 valeurs à transmettre, correspondant à exactement une période du signal *)
        SP5.ParametreEmission( sortieUtilisee, per/100, 100);
   (* l'intervalle de temps deltat entre deux valeurs émises est le 100eme de la période, arrondi au multiple de 200ns le plus proche *)
   (* 2. Remplissage du tableau *)
        for k := 0 to 100 - 1 do
                 begin
                      t := k * per/100;
                      u := ampl * sin( 2 * pi * t / per + phase);
                      SP5.PlaceValeur(sortieUtilisee, k, u);
                 end;  (* for k := 0 .... *)
    (* 3. transmission des valeurs à SYSAM *)
        SP5.TransmetValeurs  (SortieUtilisee);
    (* 4. Déclenchement de l'émission *)
        SP5.DeclencheEmission( SortieUtilisee);


end;

procedure TForm1.BEmissionUContinueClick(Sender: TObject);
var V1 : single ; vNum1 : word;
begin
   memo1.lines.add( 'Emission d’’une tension continue de 2,00 V ');

{Choix de la valeur de la tension continue à émettre}
   V1 := 2.00 ;

{Calcul de la valeur entière correspondante }
   Vnum1 := round( 2047 * (1 + V1 / 10) );

{Emission de la valeur :}
   SP5.EmissionValContinue( SA1 , vNum1 );

end;

procedure TForm1.BlectureBologicClick(Sender: TObject);
var  s1: string;
     ValBin: integer;


begin
   memo1.lines.add(' ');
   memo1.lines.add('Lecture de l''état des ports logiques B4 et B7');
   valBin := SP5.litPLB(4);
   s1 := '   B4 : ' + intToStr(valBin);
   memo1.lines.add(s1);
   valBin := SP5.litPLB(7);
   s1 := '   B7 : ' + intToStr(valBin);
   memo1.lines.add(s1);

end;

procedure TForm1.TrackBar1Change(Sender: TObject);
var  U1   : single;
begin
 { calcul de la valeur de tension à émettre en fonction de la position du curseur}
  U1 := Trackbar1.position/10 ;
{ affichage de la tension émise }
  edit1.text := FloatToStrF(U1, ffGeneral, 3 , 2);
{ actualisation de l’affichage }
  edit1.update;
{ émission }
  SP5.EmissionUContinue( SA1 , U1);

end;

initialization
  {$I unit1.lrs}

end.

